<?php

		require_once("../Util/Constant.php");
		require_once(CONST_DIR_DATA_PROTECTION()."/Util/Guid.php");
		require_once(CONST_DIR_DATA_PROTECTION()."/Util/String.php");

		////////////////////////////////////////////////////////
		//
		// This is the fundamental base class of all controls.
		// All controls should inherit from this class
		//
		class Control 
		{
			var $Parent 	= 'string';
			var $Name 		= 'string';
			var $Mode_		= 'normal';
			var $Text		= 'string';
			var $Enabled 	= true;
			var $Visible 	= true;
			var $Width 		= -1;
			var $Height		= -1;
			var $IsInitialized = false;
			
			function Control ($name, $parent)
			{
				$EnumMode = new Mode();
				
				$this->Name  = strreplaceall( $name, " ", "_" );
				$this->Parent = $parent;

				$this->Mode_ = $EnumMode->NORMAL;
				
			}

			function IsPostBack()
			{
				$referrer = $_SERVER['HTTP_REFERER'];
				$self = $_SERVER['PHP_SELF'];
				
				if ( isset( $referrer ) && trim( $referrer ) != ""  && StrEndsWith ( $referrer, $self )==true )
					return true;
				else 
					return false;
			}
			
			function FullName()
			{
				if ( isset( $this->Parent ) == false || $this->Parent == null )
					return $this->Name."_"."root";
				else
					return $this->Name."_".$this->Parent->FullName();
			}

			//
			// Alias to FuillName() function. May be overridden by derived control
			//
			function InstanceName ()
			{
				return $this->FullName();
			}
		
			function Mangle()
			{
				print ( $this->InstanceName() );
			}
		

			function Display()
			{
				if ( $this->IsInitialized == false )
				{
					$this->__protected__OnInit();
					$this->IsInitialized = true;
				}
				
				$this->__private__Process();
			}
			 
			function __private__Process()
			{
				if ( $this->Visible == false )
					return;
					
				$EnumMode = new Mode();
				
				//
				// Request Processing 
				// Getting data from query string to this control
				//
				$this->__protected__OnRequest();
				
				//
				// Intermediate Processing
				//
				$this->__protected__OnProcess();
				
				//
				// Response Processing
				// Writing data from control to response (in html format)
				//
				$this->__protected__OnResponse();
			}

			//////////////////////////////////////////////////
			//
			// 		OnInit - This is where 'initialization' 
			//			takes place. 
			//
			function __protected__OnInit()
			{
				// BUG FIX : It seems little weird as why Init() call is placed
				// on several places such as Display, Request and Resposne.
				// Reason is that calling Init() in constructor cause components
				// to be initialized but they are lost and not functional.
				// Display() method is not guaranteed to be called. Sometimes,
				// this method 'OnRequest()' is called directly. 
				// However, it is safe to assume that OnRequest will be called
				// before doing anything.
				// As a solution, Init() method can be called in Display(),
				// OnRequest() or OnResponse() methods by checking if
				// OnInit() has not already being called by checking
				// state of variable $this->IsInitialize();

			}
			
			//////////////////////////////////////////////////
			//
			// 		On Request - This is where 'Request' is
			//			processed. Here syncronization of
			//			controls should take place from Request
			//			to variables.
			//		Code Note : (CAUTION) Very important point for derived classes.
			//			The OnRequest() method should be the first one, in 
			//			derived class and any other statement after ward, 
			//			because Request() method extracts variables it's
			//			variables from session such as Mode, Visible, Enabled
			//			etc. If OnRequest() is called after any of these variables
			//			are modified, this method will override the modifications
			//			and populate it with old values from session. Hence it is
			//			extremely important that this is the first method in dervied
			//			class's override method, to make sure that user modifications
			//			are not over-written
			function __protected__OnRequest()
			{
				if ( $this->IsInitialized == false )
				{
					$this->__protected__OnInit();
					$this->IsInitialized = true;
				}
				
				if ( $this->IsPostBack() == true )
				{
					// $this->Name 	= $_SESSION[ $this->FullName().'Name']; (- No need to attach and detach
					// $this->Parent 	= $_SESSION[ $this->FullName().'Parent' ]; // BUG : absense of parent (null), cause browser to go into crash
					if ( isset ($_SESSION[ $this->FullName().'Mode']) )
						$this->Mode_ 	= $_SESSION[ $this->FullName().'Mode'];
					if ( isset ( $_SESSION[ $this->FullName().'Visible'] ))
						$this->Visible 	= $_SESSION[ $this->FullName().'Visible'];
					if ( isset ($_SESSION[ $this->FullName().'Enabled' ]))	
						$this->Enabled 	= $_SESSION[ $this->FullName().'Enabled' ];
					if ( isset ($_SESSION[ $this->FullName().'Width' ]) )
						$this->Width   	= $_SESSION[ $this->FullName().'Width' ];
					if ( isset ($_SESSION[ $this->FullName().'Hight' ]) )
						$this->Height  	= $_SESSION[ $this->FullName().'Hight' ];
				}

				$EnumMode = new Mode();

				switch ( strtoupper($this->Mode_) )
				{
					case $EnumMode->NORMAL 	: $this->__protected__OnRequestNormal();
						break;
					case $EnumMode->SELECT 	: $this->__protected__OnRequestSelect();
						break;
					case $EnumMode->EDIT 	: $this->__protected__OnRequestEdit();
						break;
				}
			}
			
			function __protected__OnRequestNormal()
			{
			}

			function __protected__OnRequestSelect()
			{
			}

			function __protected__OnRequestEdit()
			{
			}

			//////////////////////////////////////////////////
			//
			// 		On Process - This is where drawing
			//		and displaying logic should reside
			//
			function __protected__OnProcess()
			{
				$EnumMode = new Mode();

				switch ( strtoupper($this->Mode_) )
				{
					case $EnumMode->NORMAL 	: $this->__protected__OnProcessNormal();
						break;
					case $EnumMode->SELECT 	: $this->__protected__OnProcessSelect();
						break;
					case $EnumMode->EDIT 	: $this->__protected__OnProcessEdit();
						break;
					default:
					{
						// print ( "No Mode specificed for control");
					}
				}
			}

			function __protected__OnProcessNormal()
			{
			}

			function __protected__OnProcessSelect()
			{
			}

			function __protected__OnProcessEdit()
			{
			}
			
			//////////////////////////////////////////////////
			//
			// 		On Response - This is where 'Response' is
			//			processed. Here syncronization of
			//			controls should take place from variables
			//			to Response.
			//
			function __protected__OnResponse()
			{
				if ( $this->IsInitialized == false )
				{
					$this->__protected__OnInit();
					$this->IsInitialized = true;
				}

				if ( $this->Visible == false )
					return;
					
				$EnumMode = new Mode();

				switch ( strtoupper($this->Mode_) )
				{
					case $EnumMode->NORMAL 	: $this->__protected__OnResponseNormal();
						break;
					case $EnumMode->SELECT 	: $this->__protected__OnResponseSelect();
						break;
					case $EnumMode->EDIT 	: $this->__protected__OnResponseEdit();
						break;
				}
				
				// $_SESSION[ $this->FullName().'Parent' ] = $this->Parent;
				$_SESSION[ $this->FullName().'Mode'] 	= $this->Mode_;
				$_SESSION[ $this->FullName().'Text'] 	= $this->Text;
				$_SESSION[ $this->FullName().'Visible'] = $this->Visible;
				$_SESSION[ $this->FullName().'Enabled' ]= $this->Enabled;
				$_SESSION[ $this->FullName().'Width' ] 	= $this->Width;
				$_SESSION[ $this->FullName().'Hight' ] 	= $this->Height;
			}
			
			function __protected__OnResponseNormal()
			{
			}
			
			function __protected__OnResponseSelect()
			{
			}
			
			function __protected__OnResponseEdit()
			{
			}
			
			//
			// Need to use Globals to hold Guid object's single
			// instance, because nethier static nor global (out
			// side current document) are supported in PHP 4.3+
			// Also globals does not seem to be accessible within
			// class itself.
			//
			function __private__NewId()
			{
				return $GLOBALS['Guid']->NewId();
			}
			
			//
			//
			//
			function SetVisible ( $boolean )
			{
				$this->Visible = $boolean;
			}
			
			function GetVisible ()
			{
				return $this->Visible;
			}
		}			
		
		///////////////////////////////////////////////////////
		//
		//	Different modes, this control may have
		//
		class Mode
		{
			var $SELECT 	= "SELECT";			
			var $NORMAL 	= "NORMAL";
			var $EDIT 		= "EDIT";
		}
		
		$Mode = new Mode();
		
		///////////////////////////////////////////////////////
		//
		//	Various transitions, this control may go through
		//	while changing the mode(s)
		//
		class Transition
		{
			var $NORMAL_TO_SELECT 		= "NORMAL_TO_SELECT";			
			var $NORMAL_TO_EDIT 		= "NORMAL_TO_EDIT";
			var $SELECT_TO_EDIT	 		= "SELECT_TO_EDIT";						
			var $SELECT_TO_NORMAL	 	= "SELECT_TO_NORMAL";						
			var $EDIT_TO_NORMAL 		= "EDIT_TO_NORMAL";
			var $EDIT_TO_SELECT 		= "EDIT_TO_SELECT";
		}
		
		$Transition = new Transition();
?>
<?php
		// $c1 = new Control("my control");
		// $c1->Display();
		// print ( $c1->Name );
		// print ( "<br/>\n" );
		
		// $c2 = new Control("my control");
		// $c2->Display();
		// print ( $c2->Nmae );
		// print ( "<br/>\n" );
?>