<?php

	//-----------------------------------------------------------------------------
	//
	//	Deployment maintains information about software deployed on
	//	machine. This class maintains software deployment information and 
	//  provides interface to read and write to software deployment file 
	//	which is Deployment.xml - Imran Salim
	//
	//	FRESHNESS & ACCURACY of data held in Software Deployment
	//	has to be maintained by periodic visits to source information sources
	//	such as dataprotection.xml and visiting installed locations of software.
	//	A periodic visit can be based on delta time or specific events such
	//	as attempt to launch an application - Imran Salim
	//
	//-----------------------------------------------------------------------------
	
	require_once("../Util/Constant.php");

	session_start();
	
	require_once(CONST_DIR_DATA_PROTECTION()."/Util/Array.php");
	require_once(CONST_DIR_DATA_PROTECTION()."/Util/Platform.php");
	
	class Deployment
	{
		var $List = array();
		
		var $XmlNamespace;
		var $XmlSchemaLocation;
		
		var $ReadFilePath;
		var $WriteFilePath;

		function Deployment()
		{
			$this->XmlNamespace = CONST_XML_NAMESPACE();
			$this->XmlSchemaLocation = CONST_XML_SCHEMA_SOFTWARE_DEPLOYMENT();
			$this->ReadFilePath = CONST_XML_FILE_SOFTWARE_DEPLOYMENT();
			$this->WriteFilePath = CONST_XML_FILE_SOFTWARE_DEPLOYMENT();
			// $this->WriteFilePath = CONST_XML_OUTPUT_SOFTWARE_DEPLOYMENT(); <- An alternate for dev and experimental purposes
		}
		
		function & Create ()
		{
			$sw = new Software();
			return $sw;
		}
		
		function Add (& $software)
		{
			array_add($this->List, $software);
		}
		
		function Remove(& $software)
		{
			array_remove($this->List, & $software);
		}
		
		function & FindById ( $Id )
		{
			foreach ($this->List as $sw) 
			{
				if ( $sw->Id == $Id )
					return  $sw;
			}
			return null;
		}
		
		//
		// Why Update is important ? PHP 4, passes and assigns variables by value.
		// Passing by reference using & has been confusing and problematic WTMA syndrom
		// (Way too many ampersand syndrome). As values are passed and copies are made,
		// a simple mechanism is made to update the values back to original repsotiroy.
		// Hence Update method is devised. - Imran Salim
		//
		function Update ( & $software )
		{
			$newArray = array();
			foreach ($this->List as $sw) 
			{
				if ( $sw->Id != $software->Id )
					array_add( $newArray, $sw );
			}
			array_add( $newArray, $software );
			$this->List = & $newArray;
		}
		
		function ReadFromXml()
		{
			$dom = domxml_open_file($this->ReadFilePath);
			
			$xmlnodeRoot = $dom->document_element();
			$xmlnodeSoftwareList = $xmlnodeRoot->get_elements_by_tagname("Software");
			
			$this->__private__ReadFromXml($xmlnodeSoftwareList);
		}
		
		function __private__ReadFromXml($xmlnodeSoftwareList)
		{
			foreach ( $xmlnodeSoftwareList as $xmlnodeSoftware )
			{
				$software = new Software();
				$software->__private__ReadFromXml($xmlnodeSoftware);
				$this->Add($software);
			}
		}
		
		function WriteToXml( $printFileToScreen = false )
		{
			$xmldoc = domxml_new_doc("1.0");
			
			$root = $xmldoc->create_element("Root");
			$xmldoc->append_child( $root );
			
			$xmlattributeXmlNsXsi = $xmldoc->create_attribute("xmlns:xsi", $this->XmlNamespace);
			$root->append_child ( $xmlattributeXmlNsXsi );
			
			$xmlattributeSchemaLocation = $xmldoc->create_attribute ( "xsi:noNamespaceSchemaLocation", $this->XmlSchemaLocation );
			$root->append_child ( $xmlattributeSchemaLocation );
			
			foreach ( $this->List as $software )
			{
				$xmlnodeSoftware = $software->__private__WriteToXml($xmldoc);
				$root->append_child ( $xmlnodeSoftware );
			}

			if ( $printFileToScreen == true )
			{	// Print Xml file
				print ("<hr/>");
				print htmlentities( $xmldoc->dump_mem(true) );
				print ("<hr/>");
			}
			$xmldoc->dump_file( $this->WriteFilePath );
		}
	}

	class Software
	{
		// Attributes
		var $Id;
		
		// Tags
		
		var $Source;
		var $DownloadInfo;
		var $CmdInstall;
		var $Installation;
		
		function Software()
		{
			$this->Source = new Source();
			$this->DownloadInfo = new DownloadInfo();
			$this->CmdInstall = & new CmdInstall();
			$this->Installation = new Installation();
		}
		
		function __private__ReadFromXml($xmlnodeSoftware)
		{
			$Id = $xmlnodeSoftware->get_attribute("Id");
			$this->Id = $Id;
			
			$xmlnodeSource = $xmlnodeSoftware->get_elements_by_tagname("Source");
			// $this->Source = new Source();
			$this->Source->__private__ReadFromXml ( $xmlnodeSource[0] );

			$xmlnodeDownload = $xmlnodeSoftware->get_elements_by_tagname("Download");
			// $this->DownloadInfo = new DownloadInfo();
			$this->DownloadInfo->__private__ReadFromXml( $xmlnodeDownload[0] );

			$commandList = $xmlnodeSoftware->get_elements_by_tagname("CmdInstall");
			if ( $commandList != null && count ( $commandList ) > 0 )
			{
				$commandInstall = $commandList[0];
				$this->CmdInstall->__private__ReadFromXml ( $commandInstall );
			} 
			else { $this->CmdInstall == null; }

			$xmlnodeInstallation = $xmlnodeSoftware->get_elements_by_tagname("Installation");
			// $this->Installation = new Installation();
			$this->Installation->__private__ReadFromXml ( $xmlnodeInstallation[0] );
		}
		
		function __private__WriteToXml(& $xmldoc )
		{
			$xmlnodeSoftware = $xmldoc->create_element("Software");
			
			$xmlattributeId = $xmldoc->create_attribute("Id", $this->Id );
			$xmlnodeSoftware->append_child ( $xmlattributeId );
			
			if ( $this->Source != null )
			{
				$xmlnodeSource = $this->Source->__private__WriteToXml($xmldoc );
				$xmlnodeSoftware->append_child ( $xmlnodeSource );
			}
			
			if ( $this->DownloadInfo != null )
			{
				$xmlnodeDownloadInfo = $this->DownloadInfo->__private__WriteToXml ( $xmldoc );
				$xmlnodeSoftware->append_child ( $xmlnodeDownloadInfo );
			}

			if ( $this->CmdInstall != null )
			{
				$xmlnodeInstallCommands = $this->CmdInstall->__private__WriteToXml(& $xmldoc);
				$xmlnodeSoftware->append_child ( $xmlnodeInstallCommands );
			}
			
			if ( $this->Installation != null )
			{
				$xmlnodeInstallation = $this->Installation->__private__WriteToXml ( $xmldoc );
				$xmlnodeSoftware->append_child ( $xmlnodeInstallation );
			}
			
			return $xmlnodeSoftware;
		}
	}

	class Source 
	{
		var $CDDVD;
		var $FTP;
		
		function Source()
		{
			$this->CDDVD = new CDDVD();
			$this->FTP = new FTP();
		}
		
		function __private__ReadFromXml($xmlnodeSource)
		{
			$xmlnodeCDDVD = $xmlnodeSource->get_elements_by_tagname("CD-DVD");
			if ( count($xmlnodeCDDVD) == 1 )
			{
				// $this->CDDVD = new CDDVD();
				$this->CDDVD->__private__ReadFromXml($xmlnodeCDDVD[0]);
			}
			else if ( count($xmlnodeCDDVD) <= 0 )
				print ("WARNING : No such element present"."<br/>");
			else if ( count ($xmlnodeCDDVD ) >= 1 )
				print ("WARNING : Multiple elements present"."<br/>");
			
			$xmlnodeFTP = $xmlnodeSource->get_elements_by_tagname("FTP");
			if ( count ($xmlnodeFTP ) == 1 )
			{
				// $this->FTP = new FTP();
				$this->FTP->__private__ReadFromXml ($xmlnodeFTP[0]);
			}
			else if ( count ($xmlnodeFTP ) <= 0 )
				print ("WARNING : No such element present"."<br/>");
			else if ( count ($xmlnodeFTP) >= 1 )
				print ("WARNING : Multiple elements present"."<br/>");
		}
		
		function __private__WriteToXml( & $xmldoc )
		{
			$xmlnodeSource = $xmldoc->create_element("Source");
			
			if ( $this->CDDVD != null )
			{
				$xmlnodeCDDVD = $this->CDDVD->__private__WriteToXml( $xmldoc );
				if ( $xmlnodeCDDVD != null && $xmlnodeCDDVD != false )
					$xmlnodeSource->append_child ( $xmlnodeCDDVD );
			}
				
			if ( $this->FTP != null )
			{
				$xmlnodeFTP = $this->FTP->__private__WriteToXml( $xmldoc );
				if ( $xmlnodeFTP != null && $xmlnodeFTP != false )
					$xmlnodeSource->append_child ( $xmlnodeFTP );
			}
			
			return $xmlnodeSource;
		}
	}
	
	class CDDVD
	{
		var $Label;
		var $Path;
		var $InstallFile;
		
		function CDDVD()
		{
		}
		
		function __private__ReadFromXml($xmlnodeCDDVD)
		{
			$this->Label = $xmlnodeCDDVD->get_attribute("label");
			$this->Path = $xmlnodeCDDVD->get_attribute("path");
			$this->InstallFile = $xmlnodeCDDVD->get_attribute("filename");
		}
		
		function __private__WriteToXml( & $xmldoc )
		{
			$xmlnodeCDDVD = $xmldoc->create_element ("CD-DVD");
			
			$xmlattributeLabel = $xmldoc->create_attribute("label", $this->Label);
			$xmlnodeCDDVD->append_child ( $xmlattributeLabel );
			
			$xmlattributePath = $xmldoc->create_attribute("path", $this->Path);
			$xmlnodeCDDVD->append_child ( $xmlattributePath );
			
			$xmlattributeFilename = $xmldoc->create_attribute("filename", $this->InstallFile);
			$xmlnodeCDDVD->append_child ( $xmlattributeFilename );
			
			return $xmlnodeCDDVD;
		}
	}
	
	class FTP
	{
		var $Url;
		
		var $Username;
		var $Password;

		var $Path;
		var $Filename;
		var $RemoteSizeMax; // Populated from InfoFtp.php by Manager of Software Deployment
		
		var $ExistInternetConnection = false; // Populated from InfoFtp.php by Manager of Software Deployment
		var $ExistRemoteFile = false; // Populated from InfoFtp.php by Manager of Software Deployment

		function FTP()
		{
		}
		
		function __private__ReadFromXml($xmlnodeFTP)
		{
			$this->Url = $xmlnodeFTP->get_attribute("url");

			$this->Username = $xmlnodeFTP->get_attribute("username");
			$this->Password = $xmlnodeFTP->get_attribute("password");
			
			$this->Path = $xmlnodeFTP->get_attribute("path");
			$this->Filename = $xmlnodeFTP->get_attribute("filename");
			$this->RemoteSizeMax = $xmlnodeFTP->get_attribute("remotesizemax");
			
			$this->LastAccessed = $xmlnodeFTP->get_attribute("lastaccessed");
			$this->LastUpdated= $xmlnodeFTP->get_attribute("lastupdated");
			
			$this->ExistInternetConnection = $xmlnodeFTP->get_attribute("existinternetconnection");
			$this->ExistRemoteFile = $xmlnodeFTP->get_attribute("existremotefile");
			
			$this->ExistInternetConnection = $this->ExistInternetConnection == "true" ? true : false;
			$this->ExistRemoteFile = $this->ExistRemoteFile == "true" ? true : false;
		}
		
		function __private__WriteToXml( & $xmldoc )
		{
			$xmlnodeFTP = $xmldoc->create_element("FTP");
			
			$xmlattributeUrl = $xmldoc->create_attribute("url", $this->Url);
			$xmlnodeFTP->append_child ( $xmlattributeUrl );
			
			$xmlattributePath = $xmldoc->create_attribute("path", $this->Path );
			$xmlnodeFTP->append_child ( $xmlattributePath );
			
			$xmlattributeFilename = $xmldoc->create_attribute("filename", $this->Filename );
			$xmlnodeFTP->append_child ( $xmlattributeFilename );
			
			$xmlattributeRemoteSizeMax = $xmldoc->create_attribute("remotesizemax", $this->RemoteSizeMax );
			$xmlnodeFTP->append_child ( $xmlattributeRemoteSizeMax );
			
			$xmlattributeUsername = $xmldoc->create_attribute("username", $this->Username );
			$xmlnodeFTP->append_child ( $xmlattributeUsername );
			
			$xmlattributePassword = $xmldoc->create_attribute("password", $this->Password );
			$xmlnodeFTP->append_child ( $xmlattributePassword );
			
			$xmlattributeLastAccessed = $xmldoc->create_attribute("lastaccessed", $this->LastAccessed);
			$xmlnodeFTP->append_child ( $xmlattributeLastAccessed );
			
			$xmlattributeLastUpdated = $xmldoc->create_attribute("lastupdated", $this->LastUpdated );
			$xmlnodeFTP->append_child ( $xmlattributeLastUpdated );
			
			$sExistInternetConnection = $this->ExistInternetConnection == true ? "true" : "false";
			$xmlattributeExistInternetConnection = $xmldoc->create_attribute("existinternetconnection", $sExistInternetConnection);
			$xmlnodeFTP->append_child ( $xmlattributeExistInternetConnection );
			
			$sExistRemoteFile = $this->ExistRemoteFile == true ? "true" : "false";
			$xmlattributeExistRemoteFile = $xmldoc->create_attribute("existremotefile", $sExistRemoteFile);
			$xmlnodeFTP->append_child ( $xmlattributeExistRemoteFile );
			
			return $xmlnodeFTP;
		}
		
		// Since accessing FTP information is expensive i.e. going over internet,
		// it's information should be updated with deligence i.e. on as needed 
		// basis. The variables below helps 'Manager of Software Deployment' 
		// in achieveing this task, by extracting remote ftp file info in
		// careful manner and not for every request - Imran
		// The 'Mark' function below, for Access and Update sets the value
		// of LastAccessed and LastUpdated to the time of calling the 'Mark'
		// function - Imran Salim
		
		var $LastAccessed = 0;
		var $LastUpdated = 0;
		
		function Accessed()
		{
			$this->LastAccessed = time();
		}
		
		function Updated()
		{
			$this->LastUpdated = time();
		}
	}

	class CmdInstall
	{
		var $UnZip = false;
		var $UnTar = false;
		var $RpmInstall = false;
		var $DirExtract = "";
		var $SetupFile = "";
		var $OS;

		function CmdInstall()
		{
			$this->OS = OSClientPlatform();
		}
		
		function __private__ReadFromXml ( $xmlnodeCommand )
		{
			$unzip				= $xmlnodeCommand->get_attribute("unzip");
			$untar 				= $xmlnodeCommand->get_attribute("untar");
			$rpminstall 		= $xmlnodeCommand->get_attribute("rpminstall");
			
			$this->UnZip 		= $unzip=="true"?true:false;
			$this->UnTar 		= $untar=="true"?true:false;
			$this->RpmInstall 	= $rpminstall=="true"?true:false;
			$this->DirExtract	= $xmlnodeCommand->get_attribute("dirextract");
			$this->SetupFile	= $xmlnodeCommand->get_attribute("setupfile");
			$this->OS 			= $xmlnodeCommand->get_attribute("os");
			
			// @ print ( "UnZip = ".$this->UnZip." but xml's unzip = ".$unzip."<br/>");
			// @ print ( "UnTar = ".$this->UnTar." but xml's untar = ".$untar."<br/>");
			// @ print ( "RpmInstall = ".$this->RpmInstall." but xml's rpminstall = ".$rpminstall."<br/>");
		}
	
		function __private__WriteToXml( & $xmldoc )
		{
			$unzip =  ( $this->UnZip == true ) ? "true":"true";
			$untar = ( $this->UnTar == true ) ? "true":"false";
			$rpminstall = ( $this->RpmInstall == true ) ? "true":"false";
			
			$xmlnodeCommands = $xmldoc->create_element ( "CmdInstall" );
			
			$xmlattributeUnZip = $xmldoc->create_attribute("unzip", $unzip );
			$xmlnodeCommands->append_child ( $xmlattributeUnZip );
			
			$xmlattributeUnTar = $xmldoc->create_attribute("untar", $untar );
			$xmlnodeCommands->append_child ( $xmlattributeUnTar );

			$xmlattributeRpmInstall = $xmldoc->create_attribute("rpminstall", $rpminstall );
			$xmlnodeCommands->append_child ( $xmlattributeRpmInstall );

			$xmlattributeDirExtract = $xmldoc->create_attribute("dirextract", $this->DirExtract );
			$xmlnodeCommands->append_child ( $xmlattributeDirExtract );

			$xmlattributeSetupFile = $xmldoc->create_attribute("setupfile", $this->SetupFile );
			$xmlnodeCommands->append_child ( $xmlattributeSetupFile );

			$xmlattributeOS = $xmldoc->create_attribute("os", $this->OS );
			$xmlnodeCommands->append_child ( $xmlattributeOS );
			
			return  $xmlnodeCommands;
		}
	}

	class Installation
	{
		var $MachineName;
		var $InstallLocation;
		var $ExecutablePath;
		var $ExecutableFile;
		var $SetupLogFilePath;
		var $ParentSoftware;
		var $State = '';
		var $StateTimestamp = 0;
		
		function Installation()
		{
			$this->State = DEPLOYMENT_STATE_INITIAL();
		}
		
		function __private__ReadFromXml($xmlnodeInstallation)
		{
			$this->MachineName 		= $xmlnodeInstallation->get_attribute("machinename");
			$this->InstallLocation 	= $xmlnodeInstallation->get_attribute("installlocation");
			$this->ExecutablePath 	= $xmlnodeInstallation->get_attribute("executablepath");
			$this->ExecutableFile 	= $xmlnodeInstallation->get_attribute("executablefile");
			$this->SetupLogFilePath = $xmlnodeInstallation->get_attribute("setuplogpath");
			$this->State 		  	= $xmlnodeInstallation->get_attribute("state");
			$this->StateTimestamp 	= $xmlnodeInstallation->get_attribute("statetimestamp");
		}
		
		function __private__WriteToXml(& $xmldoc )
		{
			$xmlnodeInstallation = $xmldoc->create_element ( "Installation" );
			
			$xmlattributeMachineName = $xmldoc->create_attribute("machinename", $this->MachineName);
			$xmlnodeInstallation->append_child ( $xmlattributeMachineName );
			
			$xmlattributeInstallLocation = $xmldoc->create_attribute("installlocation", $this->InstallLocation);
			$xmlnodeInstallation->append_child ($xmlattributeInstallLocation);
			
			$xmlattributeExecutablePath = $xmldoc->create_attribute("executablepath", $this->ExecutablePath);
			$xmlnodeInstallation->append_child ($xmlattributeExecutablePath);
			
			$xmlattributeExecutableFile = $xmldoc->create_attribute("executablefile", $this->ExecutableFile);
			$xmlnodeInstallation->append_child ($xmlattributeExecutableFile);
			
			$xmlattributeSetupLogFilePath = $xmldoc->create_attribute("setuplogpath", $this->SetupLogFilePath);
			$xmlnodeInstallation->append_child ($xmlattributeSetupLogFilePath);

			$xmlattributeState = $xmldoc->create_attribute("state", $this->State );
			$xmlnodeInstallation->append_child ($xmlattributeState);

			$xmlattributeStateTimestamp = $xmldoc->create_attribute("statetimestamp", $this->StateTimestamp );
			$xmlnodeInstallation->append_child ($xmlattributeStateTimestamp);

			return $xmlnodeInstallation;
		}

		function SetState ( $state )
		{
			$this->State = strtolower( $state );
			$this->StateTimestamp = time();
		}
		
		function GetState ()
		{
			return $this->State;
		}
		
		function ExecutableFullPath ()
		{
			return $this->ExecutablePath."/".$this->ExecutableFile;
		}
	}

	class DownloadInfo
	{
		var $LocalPath = '';
		var $LocalFile = '';
		var $State = '';
		var $StateTimestamp = 0;
		var $ProcessId = -1;
		var $HistoryManager;
		
		function DownloadInfo()
		{
			$this->HistoryManager = new HistoryManager();
			$this->State = DEPLOYMENT_STATE_INITIAL();
		}

		function SetState ( $state )
		{
			$this->State = strtolower( $state );
			$this->StateTimestamp = time();
		}
		
		function GetState ()
		{
			return $this->State;
		}
		
		function __private__ReadFromXml($xmlnodeDownloadInfo)
		{
			$this->LocalPath = $xmlnodeDownloadInfo->get_attribute("localpath");
			$this->LocalFile = $xmlnodeDownloadInfo->get_attribute("localfile");

			$this->State = $xmlnodeDownloadInfo->get_attribute("state");
			$this->StateTimestamp = $xmlnodeDownloadInfo->get_attribute("statetimestamp");
			$this->ProcessId = $xmlnodeDownloadInfo->get_attribute("processid");
			
			$xmlnodeHistoryList = $xmlnodeDownloadInfo->get_elements_by_tagname("History");
			$this->HistoryManager->__private__ReadFromXml( $xmlnodeHistoryList );
		}
		
		function __private__WriteToXml(& $xmldoc )
		{
			$xmlnodeDownloadInfo = $xmldoc->create_element("Download");
			
			$xmlattributeLocalPath = $xmldoc->create_attribute("localpath", $this->LocalPath);
			$xmlnodeDownloadInfo->append_child ( $xmlattributeLocalPath );
			
			$xmlattributeLocalFile = $xmldoc->create_attribute("localfile", $this->LocalFile);
			$xmlnodeDownloadInfo->append_child ( $xmlattributeLocalFile );
			
			$xmlattributeState = $xmldoc->create_attribute("state", $this->State);
			$xmlnodeDownloadInfo->append_child ( $xmlattributeState );
			
			$xmlattributeStateTimestamp = $xmldoc->create_attribute("statetimestamp", $this->StateTimestamp);
			$xmlnodeDownloadInfo->append_child ( $xmlattributeStateTimestamp );
			
			$xmlattributeProcesId = $xmldoc->create_attribute("processid", $this->ProcessId);
			$xmlnodeDownloadInfo->append_child ( $xmlattributeProcesId );
			
			$xmllistHistory = $this->HistoryManager->__private__WriteToXml ( $xmldoc );
			foreach ( $xmllistHistory as $xmlHistory )
				$xmlnodeDownloadInfo->append_child ( $xmlHistory );
			
			return $xmlnodeDownloadInfo;
		}

		//
		// Returns full local file path
		//
		function LocalPathFull ()
		{
			return $this->LocalPath."/".$this->LocalFile;
		}
		
		// 
		// Exist Local File
		// 
		function ExistLocalFile()
		{
			return file_exists( $this->LocalPathFull() );
		}
		
		//
		// Returns local file size
		//
		function LocalSizeCurrent()
		{
			// Hack : Bug Fix : If size of same file is continuously
			// extracted, the consecutive requests does not report
			// any new changes. A workaround is to extract size of
			// a different file, on each consecutive request. Hence,
			// in line below, the size of currently executing script
			// is extracted and then size of currently downloading 
			// file. This helps updating the new size of currently
			// downloading file. - Imran Salim
			filesize( __FILE__ /* name of this script */ );
			if ( file_exists( $this->LocalPathFull() )==true )
			{
				return filesize($this->LocalPathFull() );
			}
			else 
			{
				return 0;
			}
		}
	}
	
	class HistoryItem
	{
		var $DownloadTime;
		var $SizeTotal;
		var $SizeDownloaded;
		var $Date;
		var $State = 'None';
		var $ProcessId = -1;
		
		function HistoryItem()
		{
		}
		
		function __private__ReadFromXml($xmlnodeHistoryItem)
		{
			$this->DownloadTime = $xmlnodeHistoryItem->get_attribute("downloadtime");
			$this->SizeTotal = $xmlnodeHistoryItem->get_attribute("sizetotal");
			$this->SizeDownloaded = $xmlnodeHistoryItem->get_attribute("sizedownloaded");
			$this->Date = $xmlnodeHistoryItem->get_attribute("date");
			$this->State = $xmlnodeHistoryItem->get_attribute("state");
			$this->ProcessId = $xmlnodeHistoryItem->get_attribute("processid");
		}
		
		function __private__WriteToXml(& $xmldoc )
		{
			$xmlnodeHistory = $xmldoc->create_element("History");	
			
			$xmlattributeDownloadTime = $xmldoc->create_attribute ("downloadtime", $this->DownloadTime);
			$xmlnodeHistory->append_child ($xmlattributeDownloadTime);
			
			$xmlattributeSizeTotal = $xmldoc->create_attribute ("sizetotal", $this->SizeTotal);
			$xmlnodeHistory->append_child ($xmlattributeSizeTotal);
			
			$xmlattributeSizeDownloaded = $xmldoc->create_attribute ("sizedownloaded", $this->SizeDownloaded);
			$xmlnodeHistory->append_child ($xmlattributeSizeDownloaded);
			
			$xmlattributeDate = $xmldoc->create_attribute ("date", $this->Date);
			$xmlnodeHistory->append_child ($xmlattributeDate);
			
			$xmlattributeState = $xmldoc->create_attribute ("state", $this->State);
			$xmlnodeHistory->append_child ($xmlattributeState);
			
			$xmlattributeProcessId = $xmldoc->create_attribute ("processid", $this->ProcessId);
			$xmlnodeHistory->append_child ($xmlattributeProcessId);
			
			return $xmlnodeHistory;
		}
	}
	
	class HistoryManager
	{
		var $List = array();
		
		function HistoryManager()
		{
		}

		function & Create ()
		{
			$historyItem = new HistoryItem();
			return $historyItem;
		}
		
		function Add ( & $historyItem )
		{
			array_add($this->List, $historyItem );
		}
		
		function Remove ( & $historyItem )
		{
			array_remove( $this->List, $historyItem );
		}
		
		function __private__ReadFromXml($xmlnodeHistoryList)
		{
			foreach ( $xmlnodeHistoryList as $xmlnodeHistoryItem )
			{
				$historyItem = new HistoryItem();
				$historyItem->__private__ReadFromXml($xmlnodeHistoryItem);
				
				$this->Add( $historyItem);
			}	
		}
		
		function __private__WriteToXml(& $xmldoc )
		{
			$xmllistHistory = array();
			
			foreach ( $this->List as $history )
			{
				$xmlHistory = $history->__private__WriteToXml ( $xmldoc );
				array_add( $xmllistHistory, $xmlHistory );
			}
			
			return  $xmllistHistory;
		}
	}
	
?>

<!-- ----------------------------------------------------------------------
---------------------------------------------------------------------------
				States 	
---------------------------------------------------------------------------
----------------------------------------------------------------------- -->

<?php
 
	function DEPLOYMENT_STATE_INITIAL()
	{
		return strtolower("Initial");		
	}
	
	function DEPLOYMENT_STATE_DOWNLOAD_IN_PROGRESS()
	{
		return strtolower("DownloadInProgress");
	}
	
	function DEPLOYMENT_STATE_DOWNLOAD_SUCCESSFUL()
	{
		return strtolower("DownloadSuccessful");
	}

	function DEPLOYMENT_STATE_DOWNLOAD_FAILED()
	{
		return strtolower("DownloadFailed");
	}
	
	function DEPLOYMENT_STATE_DOWNLOAD_FROZEN()
	{
		return strtolower("DownloadFrozen");
	}
	
	function DEPLOYMENT_STATE_INSTALL_IN_PROGRESS()
	{
		return strtolower("InstallInProgerss");
	}
		
	function DEPLOYMENT_STATE_INSTALL_SUCCESSFUL()
	{
		return strtolower("InstallSuccessful");
	}
		
	function DEPLOYMENT_STATE_INSTALL_FROZEN()
	{
		return strtolower("InstallFrozen");
	}
	
	function DEPLOYMENT_STATE_INSTALL_FAILED()
	{
		return strtolower("InstallFailed");
	}
	
	
	// X function DEPLOYMENT_STATE_DEPLOYED()
	// X {
	// X 	return strtolower("Deployed");
	// X }

	// X function DEPLOYMENT_STATE_UNDEPLOYABLE()
	// X {
	// X 	return strtolower("Undeployable"); // not enough info to download and install
	// X }

?>

<?php

	///////////////////////////////////////////////////////////////////////////////
	//
	//				Test Area
	//
	function UTestDeployment()
	{
		$deployment = new Deployment();
		print ("<hr/>");
	
		$deployment->ReadFromXml();
		print ("<hr/>");
	
		$software = & $deployment->FindById("LTT_LOCAL_REQUEST");
		$software->Source->FTP->Filename = "coolio2.exe";
		$software->DownloadInfo->LocalPath = "c:\\temp";
	
		$software = $deployment->Create();
		$software->Id = "LoveId";
		$deployment->Add( $software );
		
		$software = & $deployment->FindById("LoveId");
		$software->Source->FTP->Filename = "mylove.exe";
		$software->DownloadInfo->LocalPath = "c:\\temp";
		$software->DownloadInfo->LocalFile = "good.exe";
		$software->Installation->ExecutablePath = "c:\\program files\\software";
		$software->Installation->ExecutableFile = "runme.exe";
		$deployment->Update ( $software );
		
		$deployment->WriteToXml();
	}
	
	// UTestDeployment();
	
?>