<?php

	require_once("../Util/Constant.php");
	
	// BUG FIX : Since, Deployment object is serialized and deserialized,
	//		it has to be before session_start()
	require_once(CONST_DIR_DATA_PROTECTION()."/Server/Deployment.php");
	
	session_start();
	
	require_once(CONST_DIR_DATA_PROTECTION()."/Server/InfoFtp.php");
	require_once(CONST_DIR_DATA_PROTECTION()."/Server/SetupLog.php");

?>

<!-- ----------------------------------------------------------------------
---------------------------------------------------------------------------
		Software Deployment Related APIs
---------------------------------------------------------------------------
 ---------------------------------------------------------------------- -->
 
<?php

	//
	// Code Note - This is how to get $software from API
	// $software = SwDeployMANAGER( $sectionId );
	//
	
	function SwDeployIsDeployable ( $software )
	{
		if ( $software == null )
			return false;
		else 
			return true;
	}

	function SwDeployIsStateDownloadInitial ( $software )
	{
		if ( $software == null )
			return false;
		
		$isSetupLogPresent = file_exists( $software->Installation->SetupLogFilePath );
		if ( $software->DownloadInfo->ExistLocalFile() == false /* && $isSetupLogPresent == false */ )
		{
			$software->DownloadInfo->State = DEPLOYMENT_STATE_INITIAL();
			$software->Installation->SetState ( DEPLOYMENT_STATE_INITIAL() );
			SwDeploySave( $software );
			
			return true; // if file does not exist, it is initial state
		}
		else
			return false; // if file exist, it is NOT initial state 
	}
	
	function SwDeployIsStateDownloadInProgress ( $software )
	{
		if ( $software == null )
			return false;

		if ( $software->DownloadInfo->ExistLocalFile() == true && $software->DownloadInfo->LocalSizeCurrent() >= 0 && $software->DownloadInfo->LocalSizeCurrent() < $software->Source->FTP->RemoteSizeMax )
		{
			$software->DownloadInfo->State = DEPLOYMENT_STATE_DOWNLOAD_IN_PROGRESS();
			SwDeploySave( $software );
			
			return true;
		}
		else 
			return false;
	}

	function SwDeployIsStateDownloadSuccessful ( $software ) // Uses ftp and internet connection to verify size of downloaded local file is same as size of remote file on ftp
	{
		//
		// If no $software, then no further determination can be made
		//
		if ( $software == null )
			return false;
		
		//
		// If internet connection exist, do more definitive checking by comparing local and remote size
		// 
		
		$localSize = $software->DownloadInfo->LocalSizeCurrent();
		$remoteSize = $software->Source->FTP->RemoteSizeMax;

		if ( $software->Source->FTP->ExistInternetConnection==true && $software->DownloadInfo->LocalSizeCurrent() == $software->Source->FTP->RemoteSizeMax && 
			/* Bug Fix ( Preventetive ) : if local size == 0 and for somereason, remote size is not determined, then it is better 
				to check remote size and local size to be 'set' and 'not equals to zero'. This will prevent false positives */
			isset ( $remoteSize ) && $remoteSize != 0 && isset ( $localSize  ) && $localSize != 0 )
		{
			$software->DownloadInfo->State = DEPLOYMENT_STATE_DOWNLOAD_SUCCESSFUL();
			SwDeploySave( $software );

			return true;
		}
		
		//
		// If internet connection is absents, this only means is to check is determine the presence of local file. In this case,
		// there are no means of checking remote file.
		//
		
		else if ( $software->Source->FTP->ExistInternetConnection==false && $software->DownloadInfo->ExistLocalFile() == true )
		{
			$software->DownloadInfo->State = DEPLOYMENT_STATE_DOWNLOAD_SUCCESSFUL();
			SwDeploySave( $software );

			return true;
		}
		else 
			return false;
	}
	
	function SwDeployIsStateDownloadExist( $software ) // This function is same as SwDeployIsStateDownloadSuccessful, except that download success is verified from software info's xml file
	{
		if ( $software->DownloadInfo->ExistLocalFile() == true /* && $software->DownloadInfo->State == DEPLOYMENT_STATE_DOWNLOAD_SUCCESSFUL() */ )
			return true;
		else 
			return false;
	}
		
	function SwDeployIsStateInstallInProgress ( $software )
	{
		if ( $software == null )
			return false;

		$isSetupLogPresent = file_exists( $software->Installation->SetupLogFilePath );
		if (  SwDeployIsStateDownloadSuccessful ( $software ) && $isSetupLogPresent==false )
		{
			$software->Installation->SetState ( DEPLOYMENT_STATE_INSTALL_IN_PROGRESS() );
			SwDeploySave( $software );
			
			return true;
			
		}
		else 
			return false;
	}
	
	function SwDeployIsStateInstallSuccessful ( $software )
	{
		if ( $software == null )
			return false;
		
		$isSetupLogPresent = file_exists( $software->Installation->SetupLogFilePath );

		$setupLog = new SetupLog( $software->Installation->SetupLogFilePath );
		$setupLog->ReadFromXml();

		$installedExecutable = $setupLog->InstallLocation;
		
		if (  SwDeployIsStateDownloadSuccessful ( $software ) && $isSetupLogPresent==true && file_exists( $installedExecutable )==true )
		{
			$software->Installation->SetState ( DEPLOYMENT_STATE_INSTALL_SUCCESSFUL() );
			$software->Installation->ExecutableFile = $setupLog->ExecutableName;			
			$software->Installation->ExecutablePath = $setupLog->ExecutablePath;
			SwDeploySave( $software );
			
			return true;
		}
		else
		{
			return false;
		}
	}

	function SwDeployFixIndeterminateStates ( $software )
	{
		$isSetupLogPresent = file_exists( $software->Installation->SetupLogFilePath );

		$setupLog = new SetupLog( $software->Installation->SetupLogFilePath );
		$setupLog->ReadFromXml();

		$installedExecutable = $setupLog->InstallLocation;
		
		if ( isset ( $installedExecutable ) && trim ( $installedExecutable ) != "" && $isSetupLogPresent==true && ! file_exists( $installedExecutable ) )
		{	// If installedExecutable path is present in setup log, but installed executable does not exist
			unlink( $software->Installation->SetupLogFilePath );
		}
	}
	
	function SwDeploySave( $software )
	{
		$deployment = new Deployment();
		$deployment->ReadFromXml();

		$deployment->Update( $software );
		$deployment->WriteToXml();
	}

	
?>

<!-- ----------------------------------------------------------------------
---------------------------------------------------------------------------
	
---------------------------------------------------------------------------
----------------------------------------------------------------------- -->

<?php

	//
	// 	Returns Software (Info) class object for Section ( $sectionId passed as parameter ) and for correct 
	//	operating system ( $os determined using API in platform.php ) from deployment xml file.
	//	FtpInfo updates return object ( software info ) on as needed basis.
	//
	//	NOTE : $updateRemoteFileInfo determines if code should fetch info from remote site.
	//		Since fetching info from remote site is expensive, therefore
	//		this parameter is turned off by default. Otherwise, it will be turned on
	//
	function SwDeployMANAGER( $sectionId, $updateRemoteFileInfo=false )
	{
		$os = OSClientPlatform();		
		
		 // seconds - used to determine when information is expired
		$expirationPeriod = CONST_XML_FTP_UPDATE_DELTA_TIME();

		//
		// Sessions not used, because saving info in session and reading back
		// from file may cause multiple objects of same item. Also, since 
		// sessions are unique to each user, multiple objects of same item
		// can exist for each user. Hence idea of using sessions is completely
		// dropped. Additionally, since session objects are persisted across
		// subsequent requests, sessions also have overhead of reading and
		// writing files. 
		// Hence, values are read from xml file and written back to xml file for
		// every request. The only overhead in performance is xml dom, but due
		// to convenience of object management and consistency, decision is made
		// to live with it. - Imran Salim
		//
		
		//-------------------------------------------------------------------------
		//
		// Get info from Software Deployment xml file
		//
		//-------------------------------------------------------------------------
		
		$deployment = new Deployment();
		$deployment->ReadFromXml();
		$software = & $deployment->FindById($sectionId);
		
		//
		// Add to section, if necessary
		//
		if ( !isset ( $software ) || $software == null )
		{
			$software = & $deployment->Create();
			$software->Id = $sectionId;
			$deployment->Add( $software );
		}
		
		$history = 	$software->DownloadInfo->HistoryManager;
		
		$deltaTime = time() - $software->Source->FTP->LastAccessed;
		
		//
		// 	The code below uses Constant.php to determine intervals of remote file update.
		//	Now status of remote file is checked on when download button is clicked. This functions
		//	checks the remote file info, if $updateRemoteFileInfo parameter is true. Expectation
		// 	is that when download button is clicked, the $updateRemoteFileInfo parameter is set 
		// 	to true, that cause code below to access remote file and update size and it's presence.
		//
		// @ print ( "delta time=".$deltaTime.", time=".time()." last accessed=".$software->Source->FTP->LastAccessed.", expiration perioud=".$expirationPeriod."<br/>");
		// if ( $deltaTime < $expirationPeriod )
		// {
		// 	$software->Source->FTP->Accessed();		
		// 	$deployment->Update( $software );
		// 	$deployment->WriteToXml();
		// 	// @ print ( "retrieving from repository - deployment.xml" );
		// 	return $software;
		// }
		//
		
		// @ print ( "retrieving fresh from Infoftp" );

		//-------------------------------------------------------------------------
		//
		// SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION 
		// 		SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION
		// SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION 
		// 		SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION SYNCHRONIZATION 
		//
		//-------------------------------------------------------------------------
		//
		// SYNCHRONIZATION OF INFORMATION FROM 'InfoFtp' to 'Deployment'
		//
		//.........................................................................
		//
		// Get from InfoFtp's xml source, update remote info and set values
		// in deployment xml file's structures.
		//
		//.........................................................................
		//
		// This is a core area of how data is retrieved and assembled !
		//
		//-------------------------------------------------------------------------
		
		$MgrDownloadInfo = new InfoSectionFtp();
		$MgrDownloadInfo->BuildFromXml( CONST_XML_FILE_DATA_PROTECTION() );

		$downloadInfo = & $MgrDownloadInfo->ArrayDownloadInfo[$sectionId];
		if ( ! isset( $downloadInfo ) || $downloadInfo == null )
		{
			print ( "Error : Could not find ftp info for Section=".$sectionId."<br/>" );
			return null;
		}
		$ftpInfo = & $downloadInfo->GetFileInfo( $os );
		if ( ! isset( $ftpInfo ) || $ftpInfo == null )
		{
			print ( "Error : Could not find ftp info for OS=".$os."<br/>" );
			return null;
		}
		
		$installInfo = $MgrDownloadInfo->ArrayInstallInfo[$sectionId];
		if ( isset( $installInfo ) && $installInfo != null )
		{
			$installCommand = $installInfo->OSInstallCommandList[ $os ];
		}
		
		//
		// This statement determines internet connection and existance of remote
		// file without going to remote ftp site. Hence it is more efficient.
		//
		// NOTE : This statement should be above $ftpInfo->UpdateRemoteFileInfo();
		//
		$ftpInfo->UpdateInfo();
		
		//
		// Expesnive in time, but not performed for every read. Only
		// called when expiration time has exceeded. See code above
		// and use of expirationPeriod - Imran Salim
		//
		// NOTE : This statement should be below $ftpInfo->UpdateInfo();
		//
		if ( $updateRemoteFileInfo == true )
		{
			$ftpInfo->UpdateRemoteFileInfo();
			$software->Source->FTP->RemoteSizeMax = $ftpInfo->RemoteSizeMax();
		}
		
		// NOTE : VERY IMPORTANT : The statements below should be below $ftpInfo->UpdateRemoteFileInfo() and $ftpInfo->UpdateInfo()
		
		//
		// Synchronization : Update Software Deployment Info from ftpInfo
		//
		$software->Source->FTP->Url 					= $ftpInfo->url;
		$software->Source->FTP->Username 				= $ftpInfo->username;
		$software->Source->FTP->Password 				= $ftpInfo->password;
		$software->Source->FTP->Path 					= $ftpInfo->remoteFilePath;
		$software->Source->FTP->Filename 				= $ftpInfo->remoteFileName;
		// $software->Source->FTP->RemoteSizeMax 			= $ftpInfo->RemoteSizeMax(); 	// NOTE : Only updated if $updateRemoteFileInfo is true. @see above if ( $updateRemoteFileInfo) statement.
																							// Without updating remote info, the fetched size will be 0 bytes and cause problems when downloading.
		$software->Source->FTP->ExistInternetConnection = $ftpInfo->ExistInternetConnection;
		$software->Source->FTP->ExistRemoteFile 		= $ftpInfo->ExistRemoteFile;
		$software->Source->FTP->Updated();				// Mark LastUpdated with current time
		$software->Source->FTP->Accessed();				// Mark Last Access with current time
				
		$downloadLocalPath = CONST_DIR_SERVER_DOWNLOAD()."/".$software->Id;
		$software->DownloadInfo->LocalPath = $downloadLocalPath;
		$software->DownloadInfo->LocalFile = $software->Source->FTP->Filename;
		
		$software->CmdInstall->UnTar 		= $installCommand->UnTar;
		$software->CmdInstall->UnZip 		= $installCommand->UnZip;
		$software->CmdInstall->RpmInstall 	= $installCommand->RpmInstall;
		$software->CmdInstall->DirExtract 	= $installCommand->DirExtract;
		$software->CmdInstall->SetupFile 	= $installCommand->SetupFile;
		$software->CmdInstall->OS 			= $installCommand->OS;
		
		$software->Installation->SetupLogFilePath 		= $downloadLocalPath."/setuplog.xml";
		
		$deployment->Update( $software );
		$deployment->WriteToXml();

		//-------------------------------------------------------------------------
		//
		//	Return value
		//
		//-------------------------------------------------------------------------
		
		return $software;
	}
	
?>

<?php
	// $software = SwDeployMANAGER( "BACKUPAPPLICATION_LOCAL_REQUEST" );
	// SwDeployInstallUpdateStatus ( $software );
	// print ("<hr/>");
	// print ("Is Install in progerss  = ". SwDeployIsStateInstallInProgress( $software )."<br/>" );
	// print ("<hr/>");
	// print ("Is Install successful   = ". SwDeployIsStateInstallSuccessful( $software )."<br/>" );
	// SwDeployMANAGER( "LTT_LOCAL_REQUEST" );
?>